# -*- coding: utf-8 -*-
import logging
import argparse
import sys
import json

from ncstorage import __version__
logger = logging.getLogger(__name__)


class RenameDict(dict):

    def __init__(self, string_dict):
        super(RenameDict, self).__init__()
        for pair in string_dict.split(','):
            key, value = pair.split(':')
            self[key] = value


def get_args():
    """Define the `esmvaltool` command line."""
    # parse command line args
    parser = argparse.ArgumentParser(
        description="Utility set to manage data storages based on NetCDF",
        formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version=__version__,
        help="return ESMValTool's version number and exit")
    subparsers = parser.add_subparsers()
    _add_compress_subcommand(subparsers)
    args = parser.parse_args()
    return args


def _add_compress_subcommand(parser):
    compress = parser.add_parser('compress', help='Compress nc file')

    shuffle = compress.add_mutually_exclusive_group(required=False)
    shuffle.add_argument(
        '-s', '--shuffle', action='store_true', dest='shuffling')
    shuffle.add_argument(
        '-ns', '--no-shuffle', action='store_false', dest='shuffling')
    compress.set_defaults(shuffling=True)

    chunkdims = compress.add_mutually_exclusive_group(required=False)
    chunkdims.add_argument(
        '-m', '--map', action='store_const', dest='chunkdims', const='map')
    chunkdims.add_argument(
        '-me', '--map_ensemble', action='store_const', dest='chunkdims',
        const='map_ensemble')
    chunkdims.add_argument(
        '-t', '--time', action='store_const', dest='chunkdims', const='time')
    chunkdims.add_argument(
        '-te', '--time_ensemble', action='store_const', dest='chunkdims',
        const='time_ensemble')
    chunkdims.add_argument(
        '-cd', '--chunk-dimension', action='append', dest='chunkdims')
    compress.set_defaults(chunkdims='map_ensemble')

    compress.add_argument('-d', '--deflate_level', default=4, type=int)
    compress.add_argument('-r', '--rename', default={}, type=RenameDict)

    compress.add_argument('input_file', help='File to compress')
    compress.add_argument(
        'output_file', help='Compressed file', nargs='?', default=None)
    compress.set_defaults(func=_compress)


def _compress(parsed):
    from ._compress import compress
    compress(
        parsed.input_file, parsed.output_file, parsed.deflate_level,
        parsed.shuffling, parsed.rename, parsed.chunkdims
    )


def run():
    """Run the `ncstorage` program, logging any exceptions."""
    args = get_args()
    try:
        args.func(args)
    except:  # noqa
        if not logger.handlers:
            # Add a logging handler if main failed to do so.
            logging.basicConfig()
        logger.exception(
            "Program terminated abnormally, see stack trace "
            "below for more information",
            exc_info=True)
        sys.exit(1)
    logger.info("Run was succesful")
